@@ -8,18 +8,315 @@
 
 import pprint
 from enum import Enum
-from tkinter import Event as tkEvent, EventType as tkEventType
+from tkinter import Event as tkEvent, EventType, EventType as tkEventType, Widget
 from typing import *
 
-from ..Json import Point
+from ..Misc import lazy_property
+from ..Names import nameof
 
 
 
 
-__all__ = ['TkinterEvent', 'lazy_property', 'Bindings', 'tkEvent']
+__all__ = ['TkinterEvent', 'Bindings', 'tkEvent']
+
+class TkinterEvent(tkEvent):
+    """
+    Event Container for the properties of an event.
+    See tkinter.Misc._substitute for event creation
+
+        Instances of this type are generated if one of the following events occurs:
+
+            For keyboard events
+
+                • KeyPress
+                • KeyRelease
+
+
+            For mouse events:
+
+                • ButtonPress
+                • ButtonRelease
+                • Motion
+                • Enter
+                • Leave
+                • MouseWheel
+                • Visibility
+                • Unmap
+                • Map
+                • Expose
+                • FocusIn
+                • FocusOut
+                • Circulate
+
+
+            For window events:
+
+                • Colormap
+                • Gravity
+                • Reparent
+                • Property
+                • Destroy
+                • Activate
+                • Deactivate
+
+        If a callback function for one of these events is registered using bind, bind_all, bind_class, or tag_bind, the callback is called with an Event as first argument.
+        It will have the following attributes (in braces are the event types for which the attribute is valid):
+
+            serial - serial number of event
+
+            num - mouse button pressed (ButtonPress, ButtonRelease)
+
+            focus - whether the window has the focus (Enter, Leave)
+
+            Height - Height of the exposed window (Configure, Expose)
+
+            Width - Width of the exposed window (Configure, Expose)
+
+            keycode - keycode of the pressed key (KeyPress, KeyRelease)
+
+            state - state of the event as a number (ButtonPress, ButtonRelease, Enter, KeyPress, KeyRelease, Leave, Motion) OR state as a string (Visibility)
+
+            time - when the event occurred
+
+            x - x-position of the mouse
+
+            y - y-position of the mouse
+
+            x_root - x-position of the mouse on the screen (ButtonPress, ButtonRelease, KeyPress, KeyRelease, Motion)
+
+            y_root - y-position of the mouse on the screen (ButtonPress, ButtonRelease, KeyPress, KeyRelease, Motion)
+
+            char - pressed character (KeyPress, KeyRelease)
+
+            send_event - see X/Windows documentation
+
+            keysym - keysym of the event as a string (KeyPress, KeyRelease)
+
+            keysym_num - keysym of the event as a number (KeyPress, KeyRelease)
+
+            type - type of the event as a number or <enum 'tkinter.EventType'>
+
+            widget - widget in which the event occurred
+
+            delta - delta of wheel movement (MouseWheel)
+
+    """
+    __slots__ = ['serial',
+                 'num',
+                 'Height',
+                 'keycode',
+                 'state',
+                 'time',
+                 'Width',
+                 'x',
+                 'y',
+                 'char',
+                 'keysym',
+                 'keysym_num',
+                 'type',
+                 'widget',
+                 'x_root',
+                 'y_root',
+                 'delta',
+                 'send_event']
+
+    serial: Union[str, int]
+    num: Optional[Union[str, int]]
+    focus: Optional[Union[int, bool]]
+    width: Optional[int]
+    height: Optional[int]
+    keycode: Optional[Union[str, int]]
+    state: int
+    time: int
+    x: Union[str, float]
+    y: Union[str, float]
+    x_root: Union[str, float]
+    y_root: Union[str, float]
+    char: Optional[str]
+    keysym: Optional[str]
+    type: EventType
+    widget: Widget
+    delta: Optional[float]
+
+    def __init__(self, source: Optional[tkEvent]):
+        super().__init__()
+        if source is not None:
+            assert (isinstance(source, tkEvent))
+            for name, value in source.__dict__.items(): setattr(self, name, value)
+
+
+    def __str__(self) -> str:
+        return f"""<{nameof(self)} ({repr(self).replace('>', '').replace('<', '')}) Object.
+{pprint.pformat(self.ToDict(), indent=4)} >"""
+
+
+    def ToDict(self) -> Dict[str, Union[int, str, float, Widget, 'Bindings']]:
+        return {
+            'KeySynonym': self.KeySynonym,
+            'IsDigit':    self.IsDigit(self.KeySynonym),
+            'IsEnter':    self.IsEnter(),
+            'IsValid':    self.IsValid(),
+            'serial':     self.serial,
+            'num':        self.num,
+            'Height':     self.height,
+            'keycode':    self.keycode,
+            'state':      self.state,
+            'time':       self.time,
+            'Width':      self.width,
+            'x':          self.x,
+            'y':          self.y,
+            'char':       self.char,
+            'keysym':     self.keysym,
+            'keysym_num': self.keysym_num,
+            'type':       self.type,
+            'widget':     self.widget,
+            'x_root':     self.x_root,
+            'y_root':     self.y_root,
+            'delta':      self.delta,
+            'send_event': self.send_event
+            }
+
+    def __call__(self, keysym: 'Bindings' = None) -> bool:
+        if keysym is not None: return self.KeySynonym == keysym
+
+        raise ValueError('Unknown value passed')
+
+    @staticmethod
+    def IsDigit(keysym: Union[int, str, 'TkinterEvent']) -> bool:
+        """
+        :param keysym:
+        :type keysym: int or str or TkinterEvent
+        :return:
+        :rtype: bool
+        """
+        if isinstance(keysym, TkinterEvent): keysym = keysym.keysym
+        return Bindings.IsDigit(keysym)
+
+    @property
+    def EventType(self) -> tkEventType:
+        return self.type
+
+
+    @lazy_property
+    def KeySynonym(self) -> 'Bindings':
+        return Bindings.FromEvent(self)
+
+
+    def IsEnter(self) -> bool:
+        return Bindings.IsEnter(self.keysym)
+
+
+    def IsValid(self):
+        return Bindings.IsKnown(self.keysym)
+
+
+    @classmethod
+    def FromShiftTabEvent(cls, event: tkEvent):
+        e = cls(event)
+        e.keysym = Bindings.ShiftTab.value
+        return e
+
+
+    @staticmethod
+    def Debug(event: tkEvent):
+        print('Debug.tkEvent', event)
+        event = TkinterEvent(event)
+        print('Debug.TkinterEvent')
+        print(str(event))
+        print(event.KeySynonym)
+
+    def Point(self) -> Optional[Tuple[int, int]]:
+        if isinstance(self.x, (float, int)) and isinstance(self.y, (float, int)): return self.x, self.y
+        return None
+
+
 
 class Bindings(Enum):
-    """ https://www.tcl.tk/man/tcl8.6/TkCmd/event.htm """
+    __doc__ = """ https://www.tcl.tk/man/tcl8.6/TkCmd/event.htm """
+    __slots__ = []
+
+    @staticmethod
+    def IsUnknown(keysym: Union[str, 'Bindings']) -> bool:
+        """
+        :param keysym:
+        :type keysym: Bindings or str
+        :return:
+        :rtype: bool
+        """
+        if isinstance(keysym, str): return keysym == Bindings.UNKNOWN.value
+
+        return keysym == Bindings.UNKNOWN
+
+    @staticmethod
+    def IsKnown(keysym: Union[str, 'Bindings']) -> bool:
+        """
+        :param keysym:
+        :type keysym: Bindings or str
+        :return:
+        :rtype: bool
+        """
+        if isinstance(keysym, str): return keysym != Bindings.UNKNOWN.value
+
+        return keysym != Bindings.UNKNOWN
+
+    @staticmethod
+    def IsDigit(keysym: Union[int, str, TkinterEvent]) -> bool:
+        """
+        :param keysym:
+        :type keysym: int or str or Bindings
+        :return:
+        :rtype: bool
+        """
+        if isinstance(keysym, TkinterEvent): keysym = keysym.keysym
+        if isinstance(keysym, Bindings): keysym = keysym.value
+
+        return keysym in ('0', '1', '2', '3', '4', '5', '6', '7', '8', '9', 'period', 'comma', ',', '.', 0, 1, 2, 3, 4, 5, 6, 7, 8, 9)
+
+    @staticmethod
+    def IsEnter(keysym: Union[str, 'Bindings']) -> bool:
+        """
+        :param keysym:
+        :type keysym: Bindings or str
+        :return:
+        :rtype: bool
+        """
+        if isinstance(keysym, str): keysym = Bindings.FromKeysym(keysym)
+
+        return keysym == Bindings.EnterKey or keysym == Bindings.KP_Enter or keysym == Bindings.Return
+
+    @staticmethod
+    def FromKeysym(keysym: str):
+        """
+        :param keysym: binding type string
+        :type keysym: str
+        :return: resulting Binding match
+        :rtype: Bindings
+        """
+        try:
+            return Bindings(keysym)
+        except ValueError as e:
+            if hasattr(Bindings, keysym): return getattr(Bindings, keysym)
+            for name in dir(Bindings):
+                if name.lower() == keysym.lower():
+                    return getattr(Bindings, name)
+
+            raise ValueError(f'Cannot find matching event: "{keysym}"') from e
+
+    @staticmethod
+    def FromEvent(event: TkinterEvent):
+        """
+        :param event: associated event
+        :type event: TkinterEvent
+        :return: resulting Binding match
+        :rtype: Bindings
+        """
+        if Bindings.IsKnown(event.keysym) and Bindings.IsKnown(event.num):
+            return Bindings.FromKeysym(f'{event.EventType.name}{event.num}')
+
+        return Bindings.FromKeysym(event.keysym)
+
+
+
     UNKNOWN = '??'
 
     # class Special(Enum):
@@ -126,6 +423,17 @@ class Bindings(Enum):
     y = 'y'
     z = 'z'
 
+    Zero = '0'
+    One = '1'
+    Two = '2'
+    Three = '3'
+    Four = '4'
+    Five = '5'
+    Size = '6'
+    Seven = '7'
+    Eight = '8'
+    Nine = '9'
+
     # class Custom(Enum):
     ShiftTab = 'Shift_Tab',
     ShiftTabEvent = '<Shift-KeyPress-Tab>'
@@ -240,263 +548,6 @@ class Bindings(Enum):
     XF86AudioMute = 'XF86AudioMute'
     XF86AudioPrev = 'XF86AudioPrev'
     XF86AudioNext = 'XF86AudioNext'
-
-
-    @staticmethod
-    def IsUnknown(keysym) -> bool:
-        """
-        :param keysym:
-        :type keysym: Bindings or str
-        :return:
-        :rtype: bool
-        """
-        if isinstance(keysym, str): return keysym == Bindings.UNKNOWN.value
-
-        return keysym == Bindings.UNKNOWN
-
-    @staticmethod
-    def IsDigit(keysym: int or str) -> bool:
-        """
-        :param keysym:
-        :type keysym: int or str or Bindings
-        :return:
-        :rtype: bool
-        """
-        if isinstance(keysym, Bindings): keysym = keysym.value
-
-        return keysym in ('0', '1', '2', '3', '4', '5', '6', '7', '8', '9', 'period', 'comma', ',', '.', 0, 1, 2, 3, 4, 5, 6, 7, 8, 9)
-
-    @staticmethod
-    def IsEnter(keysym) -> bool:
-        """
-        :param keysym:
-        :type keysym: Bindings or str
-        :return:
-        :rtype: bool
-        """
-        if isinstance(keysym, str): keysym = Bindings.FromKeysym(keysym)
-
-        return keysym == Bindings.EnterKey or keysym == Bindings.KP_Enter or keysym == Bindings.Return
-
-    @staticmethod
-    def FromKeysym(keysym: str):
-        """
-        :param keysym: binding type string
-        :type keysym: str
-        :return: resulting Binding match
-        :rtype: Bindings
-        """
-        try: return Bindings(keysym)
-        except ValueError as e:
-            if hasattr(Bindings, keysym): return getattr(Bindings, keysym)
-            for name in dir(Bindings):
-                if name.lower() == keysym.lower():
-                    return getattr(Bindings, name)
-
-            raise ValueError(f'Cannot find matching event: "{keysym}"') from e
-
-    @staticmethod
-    def FromEvent(event):
-        """
-        :param event: associated event
-        :type event: TkinterEvent
-        :return: resulting Binding match
-        :rtype: Bindings
-        """
-        assert (isinstance(event, TkinterEvent))
-        if Bindings.IsUnknown(event.keysym) and event.num != Bindings.UNKNOWN.value:
-            return Bindings.FromKeysym(f'{event.EventType.name}{event.num}')
-
-        return Bindings.FromKeysym(event.keysym)
-
-
-
-class lazy_property(object):
-    """A @property that is only evaluated once."""
-    def __init__(self, func: callable, name: str = None, doc: str = None):
-        self.__name__ = name or func.__name__
-        self.__module__ = func.__module__
-        self.__doc__ = doc or func.__doc__
-        self._func = func
-
-    def __get__(self, obj, cls=None):
-        if obj is None:
-            return self
-        value = self._func(obj)
-        setattr(obj, self._func.__name__, value)
-        return value
-
-
-class TkinterEvent(tkEvent):
-    """
-    Event Container for the properties of an event.
-    See tkinter.Misc._substitute for event creation
-
-        Instances of this type are generated if one of the following events occurs:
-
-            For keyboard events
-
-                • KeyPress
-                • KeyRelease
-
-
-            For mouse events:
-
-                • ButtonPress
-                • ButtonRelease
-                • Motion
-                • Enter
-                • Leave
-                • MouseWheel
-                • Visibility
-                • Unmap
-                • Map
-                • Expose
-                • FocusIn
-                • FocusOut
-                • Circulate
-
-
-            For window events:
-
-                • Colormap
-                • Gravity
-                • Reparent
-                • Property
-                • Destroy
-                • Activate
-                • Deactivate
-
-        If a callback function for one of these events is registered using bind, bind_all, bind_class, or tag_bind, the callback is called with an Event as first argument.
-        It will have the following attributes (in braces are the event types for which the attribute is valid):
-
-            serial - serial number of event
-
-            num - mouse button pressed (ButtonPress, ButtonRelease)
-
-            focus - whether the window has the focus (Enter, Leave)
-
-            height - height of the exposed window (Configure, Expose)
-
-            width - width of the exposed window (Configure, Expose)
-
-            keycode - keycode of the pressed key (KeyPress, KeyRelease)
-
-            state - state of the event as a number (ButtonPress, ButtonRelease, Enter, KeyPress, KeyRelease, Leave, Motion) OR state as a string (Visibility)
-
-            time - when the event occurred
-
-            x - x-position of the mouse
-
-            y - y-position of the mouse
-
-            x_root - x-position of the mouse on the screen (ButtonPress, ButtonRelease, KeyPress, KeyRelease, Motion)
-
-            y_root - y-position of the mouse on the screen (ButtonPress, ButtonRelease, KeyPress, KeyRelease, Motion)
-
-            char - pressed character (KeyPress, KeyRelease)
-
-            send_event - see X/Windows documentation
-
-            keysym - keysym of the event as a string (KeyPress, KeyRelease)
-
-            keysym_num - keysym of the event as a number (KeyPress, KeyRelease)
-
-            type - type of the event as a number or <enum 'tkinter.EventType'>
-
-            widget - widget in which the event occurred
-
-            delta - delta of wheel movement (MouseWheel)
-
-    """
-    __slots__ = ['serial', 'num', 'height', 'keycode', 'state', 'time', 'width', 'x', 'y', 'char', 'keysym', 'keysym_num', 'type', 'widget', 'x_root', 'y_root', 'delta', 'send_event']
-
-    def __init__(self, source: tkEvent = None):
-        super().__init__()
-        if source is not None:
-            assert (isinstance(source, tkEvent))
-            self.__dict__.update(source.__dict__)
-            for name, value in source.__dict__.items(): setattr(self, name, value)
-
-
-    def __str__(self) -> str: return self.ToString()
-    # def __repr__(self) -> str: return self.ToString()
-
-    def ToString(self) -> str: return f"""<{self.__class__.__name__} ({repr(self).replace('>', '').replace('<', '')}) Object.
-{pprint.pformat(self.ToDict(), indent=4)} >"""
-    def ToDict(self) -> dict:
-        """
-            {
-                'num': self.num,
-                'height': self.height,
-                'width': self.width,
-                'widget': self.widget,
-                'keysym': self.keysym,
-                'keycode': self.keycode,
-                'keysym_num': self.keysym_num,
-                'state': self.state,
-                'time': self.time,
-                'x': self.x,
-                'y': self.y,
-                'char': self.char,
-                'type': self.type,
-                'x_root': self.x_root,
-                'y_root': self.y_root,
-                'delta': self.delta,
-            }
-        :return:
-        """
-        d = self.__dict__.copy()
-        d['KeySynonym'] = self.KeySynonym
-        return d
-
-    def __enter__(self): return self
-    def __exit__(self, exc_type, exc_val, exc_tb): pass
-    def __call__(self, *, keysym: Bindings = None) -> bool:
-        if keysym is not None: return self.KeySynonym == keysym
-
-        raise ValueError('Unknown value passed')
-
-    @staticmethod
-    def IsDigit(keysym: int or str) -> bool:
-        """
-        :param keysym:
-        :type keysym: int or str or TkinterEvent
-        :return:
-        :rtype: bool
-        """
-        if isinstance(keysym, TkinterEvent): keysym = keysym.keysym
-        return Bindings.IsDigit(keysym)
-
-    @property
-    def EventType(self) -> tkEventType: return self.type
-
-    @lazy_property
-    def KeySynonym(self) -> Bindings: return Bindings.FromEvent(self)
-
-    def IsEnter(self) -> bool: return Bindings.IsEnter(self.keysym)
-
-    @staticmethod
-    def IsValid(o: str):
-        try:
-            return o != '??'
-        except (TypeError, ValueError) as e:
-            raise TypeError(f'expected str, got {type(o)}') from e
-
-    @classmethod
-    def FromShiftTabEvent(cls, event: tkEvent):
-        e = cls(event)
-        e.keysym = Bindings.ShiftTab.value
-        return e
-
-
-    @staticmethod
-    def Debug(event: tkEvent):
-        print('Debug.tkEvent', event)
-        event = TkinterEvent(event)
-        print('Debug.TkinterEvent', event.ToString())
-        print(event.KeySynonym)
-
-    def Point(self) -> Optional[Point]:
-        if isinstance(self.x, (float, int)) and isinstance(self.y, (float, int)): return Point.Create(self.x, self.y)
-        return None
+    XF86Sleep = 'XF86Sleep'
+    XF86Search = 'XF86Search'
+    XF86TouchpadToggle = 'XF86TouchpadToggle'
